@@ -22,9 +22,7 @@ function permalink_anchor($mode = 'id') {
     }
 }
 
-function get_permalink($id = false) {
-	global $post, $wpdb;
-
+function get_permalink($id = 0) {
 	$rewritecode = array(
 		'%year%',
 		'%monthnum%',
@@ -39,29 +37,24 @@ function get_permalink($id = false) {
 		'%pagename%'
 	);
 
-	if ($id) {
-		$idpost = $wpdb->get_row("SELECT ID, post_date, post_name, post_status, post_author FROM $wpdb->posts WHERE ID = $id");
-	} else {
-		$idpost = $post;
-	}
-
-	if ($idpost->post_status == 'static') {
-		return get_page_link($idpost->ID);
+	$post = & get_post($id);
+	if ($post->post_status == 'static') {
+		return get_page_link($post->ID);
 	}
 
 	$permalink = get_settings('permalink_structure');
 
-	if ('' != $permalink) {
-		$unixtime = strtotime($idpost->post_date);
+	if ('' != $permalink && 'draft' != $post->post_status) {
+		$unixtime = strtotime($post->post_date);
 
 		$category = '';
 		if (strstr($permalink, '%category%')) {
-			$cats = get_the_category($idpost->ID);
+			$cats = get_the_category($post->ID);
 			$category = $cats[0]->category_nicename;
 			if ($parent=$cats[0]->category_parent) $category = get_category_parents($parent, FALSE, '/', TRUE) . $category;
 		}
 
-		$authordata = get_userdata($idpost->post_author);
+		$authordata = get_userdata($post->post_author);
 		$author = $authordata->user_nicename;
 		$rewritereplace = 
 		array(
@@ -71,16 +64,16 @@ function get_permalink($id = false) {
 			date('H', $unixtime),
 			date('i', $unixtime),
 			date('s', $unixtime),
-			$idpost->post_name,
-			$idpost->ID,
+			$post->post_name,
+			$post->ID,
 			$category,
 			$author,
-			$idpost->post_name,
+			$post->post_name,
 		);
-		return apply_filters('post_link', get_settings('home') . str_replace($rewritecode, $rewritereplace, $permalink), $idpost);
+		return apply_filters('post_link', get_settings('home') . str_replace($rewritecode, $rewritereplace, $permalink), $post);
 	} else { // if they're not using the fancy permalink option
-		$permalink = get_settings('home') . '/?p=' . $idpost->ID;
-		return apply_filters('post_link', $permalink, $idpost);
+		$permalink = get_settings('home') . '/?p=' . $post->ID;
+		return apply_filters('post_link', $permalink, $post);
 	}
 }
 
@@ -168,9 +161,9 @@ function get_feed_link($feed='rss2') {
 		$output =  get_settings('home') . $permalink;
 	} else {
 		if ( false !== strpos($feed, 'comments_') )
-			$feed = str_replace('comments_', 'comments', $feed);
+			$feed = str_replace('comments_', 'comments-', $feed);
 
-		$output = get_settings('siteurl') . "/wp-{$feed}.php";
+		$output = get_settings('home') . "/?feed={$feed}";
 	}
 
 	return apply_filters('feed_link', $output, $feed);
@@ -205,59 +198,69 @@ function edit_comment_link($link = 'Edit This', $before = '', $after = '') {
 // Navigation links
 
 function get_previous_post($in_same_cat = false, $excluded_categories = '') {
-    global $post, $wpdb;
+	global $post, $wpdb;
 
-    if(! is_single()) {
-      return null;
-    }
+	if(! is_single()) {
+		return null;
+	}
     
-    $current_post_date = $post->post_date;
-    $current_category = $post->post_category;
+	$current_post_date = $post->post_date;
     
-    $sqlcat = '';
-    if ($in_same_cat) {
-      $sqlcat = " AND post_category = '$current_category' ";
-    }
+	$join = '';
+	if ($in_same_cat) {
+		$join = " INNER JOIN $wpdb->post2cat ON $wpdb->posts.ID= $wpdb->post2cat.post_id ";
+		$cat_array = get_the_category($post->ID);
+	 	$join .= ' AND (category_id = ' . intval($cat_array[0]->cat_ID);
+		for ($i = 1; $i < (count($cat_array)); $i++) {
+			$join .= ' OR category_id = ' . intval($cat_array[$i]->cat_ID);
+		}
+		$join .= ')'; 
+	}
 
-    $sql_exclude_cats = '';
-    if (!empty($excluded_categories)) {
-      $blah = explode('and', $excluded_categories);
-      foreach($blah as $category) {
-	$category = intval($category);
-	$sql_exclude_cats .= " AND post_category != $category";
-      }
-    }
+	$sql_exclude_cats = '';
+	if (!empty($excluded_categories)) {
+		$blah = explode('and', $excluded_categories);
+		foreach($blah as $category) {
+			$category = intval($category);
+			$sql_exclude_cats .= " AND post_category != $category";
+		}
+	}
 
-    return @$wpdb->get_row("SELECT ID, post_title FROM $wpdb->posts WHERE post_date < '$current_post_date' AND post_status = 'publish' $sqlcat $sql_exclude_cats ORDER BY post_date DESC LIMIT 1");
+	return @$wpdb->get_row("SELECT ID, post_title FROM $wpdb->posts $join WHERE post_date < '$current_post_date' AND post_status = 'publish' $sqlcat $sql_exclude_cats ORDER BY post_date DESC LIMIT 1");
 }
 
 function get_next_post($in_same_cat = false, $excluded_categories = '') {
-    global $post, $wpdb;
+	global $post, $wpdb;
 
-    if(! is_single()) {
-      return null;
-    }
+	if(! is_single()) {
+		return null;
+	}
 
-    $current_post_date = $post->post_date;
-    $current_category = $post->post_category;
+	$current_post_date = $post->post_date;
     
-    $sqlcat = '';
-    if ($in_same_cat) {
-      $sqlcat = " AND post_category = '$current_category' ";
-    }
+	$join = '';
+	if ($in_same_cat) {
+		$join = " INNER JOIN $wpdb->post2cat ON $wpdb->posts.ID= $wpdb->post2cat.post_id ";
+		$cat_array = get_the_category($post->ID);
+	 	$join .= ' AND (category_id = ' . intval($cat_array[0]->cat_ID);
+		for ($i = 1; $i < (count($cat_array)); $i++) {
+			$join .= ' OR category_id = ' . intval($cat_array[$i]->cat_ID);
+		}
+		$join .= ')'; 
+	}
 
-    $sql_exclude_cats = '';
-    if (!empty($excluded_categories)) {
-      $blah = explode('and', $excluded_categories);
-      foreach($blah as $category) {
-	$category = intval($category);
-	$sql_exclude_cats .= " AND post_category != $category";
-      }
-    }
+	$sql_exclude_cats = '';
+	if (!empty($excluded_categories)) {
+		$blah = explode('and', $excluded_categories);
+		foreach($blah as $category) {
+			$category = intval($category);
+			$sql_exclude_cats .= " AND post_category != $category";
+		}
+	}
 
-    $now = current_time('mysql');
+	$now = current_time('mysql');
     
-    return @$wpdb->get_row("SELECT ID,post_title FROM $wpdb->posts WHERE post_date > '$current_post_date' AND post_date < '$now' AND post_status = 'publish' $sqlcat $sql_exclude_cats AND ID != $post->ID ORDER BY post_date ASC LIMIT 1");
+	return @$wpdb->get_row("SELECT ID,post_title FROM $wpdb->posts $join WHERE post_date > '$current_post_date' AND post_date < '$now' AND post_status = 'publish' $sqlcat $sql_exclude_cats AND ID != $post->ID ORDER BY post_date ASC LIMIT 1");
 }
 
 function previous_post_link($format='&laquo; %link', $link='%title', $in_same_cat = false, $excluded_categories = '') {
@@ -300,139 +303,113 @@ function next_post_link($format='%link &raquo;', $link='%title', $in_same_cat =
   echo $format;	    
 }
 
+// Deprecated.  Use previous_post_link().
 function previous_post($format='%', $previous='previous post: ', $title='yes', $in_same_cat='no', $limitprev=1, $excluded_categories='') {
-    global $id, $post, $wpdb;
-    global $posts, $posts_per_page, $s;
 
-    if(($posts_per_page == 1) || is_single()) {
+	if ( empty($in_same_cat) || 'no' == $in_same_cat )
+		$in_same_cat = false;
+	else
+		$in_same_cat = true;
 
-        $current_post_date = $post->post_date;
-        $current_category = $post->post_category;
+  $post = get_previous_post($in_same_cat, $excluded_categories);
 
-        $sqlcat = '';
-        if ($in_same_cat != 'no') {
-            $sqlcat = " AND post_category = '$current_category' ";
-        }
+  if(! $post) {
+    return;
+  }
 
-        $sql_exclude_cats = '';
-        if (!empty($excluded_categories)) {
-            $blah = explode('and', $excluded_categories);
-            foreach($blah as $category) {
-                $category = intval($category);
-                $sql_exclude_cats .= " AND post_category != $category";
-            }
-        }
-
-        $limitprev--;
-        $lastpost = @$wpdb->get_row("SELECT ID, post_title FROM $wpdb->posts WHERE post_date < '$current_post_date' AND post_status = 'publish' $sqlcat $sql_exclude_cats ORDER BY post_date DESC LIMIT $limitprev, 1");
-        if ($lastpost) {
-            $string = '<a href="'.get_permalink($lastpost->ID).'">'.$previous;
-            if ($title == 'yes') {
-                $string .= wptexturize($lastpost->post_title);
-            }
-            $string .= '</a>';
-            $format = str_replace('%', $string, $format);
-            echo $format;
-        }
-    }
+	$string = '<a href="'.get_permalink($post->ID).'">'.$previous;
+	if ($title == 'yes') {
+		$string .= apply_filters('the_title', $post->post_title, $post);
+	}
+	$string .= '</a>';
+	$format = str_replace('%', $string, $format);
+	echo $format;
 }
 
+// Deprecated.  Use next_post_link().
 function next_post($format='%', $next='next post: ', $title='yes', $in_same_cat='no', $limitnext=1, $excluded_categories='') {
-    global $posts_per_page, $post, $wpdb;
-    if(1 == $posts_per_page || is_single()) {
+	
+	if ( empty($in_same_cat) || 'no' == $in_same_cat )
+		$in_same_cat = false;
+	else
+		$in_same_cat = true;
 
-        $current_post_date = $post->post_date;
-        $current_category = $post->post_category;
+  $post = get_next_post($in_same_cat, $excluded_categories);
 
-        $sqlcat = '';
-        if ($in_same_cat != 'no') {
-            $sqlcat = " AND post_category='$current_category' ";
-        }
+  if(! $post) {
+    return;
+  }
 
-        $sql_exclude_cats = '';
-        if (!empty($excluded_categories)) {
-            $blah = explode('and', $excluded_categories);
-            foreach($blah as $category) {
-                $category = intval($category);
-                $sql_exclude_cats .= " AND post_category != $category";
-            }
-        }
-
-        $now = current_time('mysql', 1);
-
-        $limitnext--;
-
-        $nextpost = @$wpdb->get_row("SELECT ID,post_title FROM $wpdb->posts WHERE post_date > '$current_post_date' AND post_date_gmt < '$now' AND post_status = 'publish' $sqlcat $sql_exclude_cats AND ID != $post->ID ORDER BY post_date ASC LIMIT $limitnext,1");
-        if ($nextpost) {
-            $string = '<a href="'.get_permalink($nextpost->ID).'">'.$next;
-            if ($title=='yes') {
-                $string .= wptexturize($nextpost->post_title);
-            }
-            $string .= '</a>';
-            $format = str_replace('%', $string, $format);
-            echo $format;
-        }
-    }
+	$string = '<a href="'.get_permalink($post->ID).'">'.$next;
+	if ($title=='yes') {
+		$string .= apply_filters('the_title', $post->post_title, $nextpost);
+	}
+	$string .= '</a>';
+	$format = str_replace('%', $string, $format);
+	echo $format;
 }
 
-function get_pagenum_link($pagenum = 1){
+function get_pagenum_link($pagenum = 1) {
 	global $wp_rewrite;
 
-   $qstr = $_SERVER['REQUEST_URI'];
+	$qstr = $_SERVER['REQUEST_URI'];
 
-   $page_querystring = "paged"; 
-   $page_modstring = "page/";
-   $page_modregex = "page/?";
-   $permalink = 0;
-	 $index = 'index.php';
+	$page_querystring = "paged"; 
+	$page_modstring = "page/";
+	$page_modregex = "page/?";
+	$permalink = 0;
 
-	 $home_root = parse_url(get_settings('home'));
-	 $home_root = $home_root['path'];
-   $home_root = trailingslashit($home_root);
-   $qstr = preg_replace('|^'. $home_root . '|', '', $qstr);
-   $qstr = preg_replace('|^/+|', '', $qstr);
+	$home_root = parse_url(get_settings('home'));
+	$home_root = $home_root['path'];
+	$home_root = trailingslashit($home_root);
+	$qstr = preg_replace('|^'. $home_root . '|', '', $qstr);
+	$qstr = preg_replace('|^/+|', '', $qstr);
 
-   // if we already have a QUERY style page string
-   if( stristr( $qstr, $page_querystring ) ) {
-       $replacement = "$page_querystring=$pagenum";
-      $qstr = preg_replace("/".$page_querystring."[^\d]+\d+/", $replacement, $qstr);
-   // if we already have a mod_rewrite style page string
-   } elseif ( preg_match( '|'.$page_modregex.'\d+|', $qstr ) ){
-      $permalink = 1;
-      $qstr = preg_replace('|'.$page_modregex.'\d+|',"$page_modstring$pagenum",$qstr);
+	$index = $_SERVER['SCRIPT_NAME'];
+	$index = preg_replace('|^'. $home_root . '|', '', $index);
+	$index = preg_replace('|^/+|', '', $index);
 
-   // if we don't have a page string at all ...
-   // lets see what sort of URL we have...
-   } else {
-      // we need to know the way queries are being written
-      // if there's a querystring_start (a "?" usually), it's deffinitely not mod_rewritten
-      if ( stristr( $qstr, '?' ) ){
-         // so append the query string (using &, since we already have ?)
-         $qstr .=  '&amp;' . $page_querystring . '=' . $pagenum;
-         // otherwise, it could be rewritten, OR just the default index ...
-      } elseif( '' != get_settings('permalink_structure')) {
-         $permalink = 1;
+	// if we already have a QUERY style page string
+	if( stristr( $qstr, $page_querystring ) ) {
+		$replacement = "$page_querystring=$pagenum";
+		$qstr = preg_replace("/".$page_querystring."[^\d]+\d+/", $replacement, $qstr);
+		// if we already have a mod_rewrite style page string
+	} elseif ( preg_match( '|'.$page_modregex.'\d+|', $qstr ) ){
+		$permalink = 1;
+		$qstr = preg_replace('|'.$page_modregex.'\d+|',"$page_modstring$pagenum",$qstr);
 
-	 // If it's not a path info permalink structure, trim the index.
-	 if (! $wp_rewrite->using_index_permalinks()) {
-	   $qstr = preg_replace("#/*" . $index . "/*#", '/', $qstr);
-	 } else {
-	   // If using path info style permalinks, make sure the index is in
-	   // the URI.
-	   if (strpos($qstr, $index) === false) {
-	     $qstr = '/' . $index . $qstr;
-	   }
-	 }
+		// if we don't have a page string at all ...
+		// lets see what sort of URL we have...
+	} else {
+		// we need to know the way queries are being written
+		// if there's a querystring_start (a "?" usually), it's definitely not mod_rewritten
+		if ( stristr( $qstr, '?' ) ){
+			// so append the query string (using &, since we already have ?)
+			$qstr .=  '&amp;' . $page_querystring . '=' . $pagenum;
+			// otherwise, it could be rewritten, OR just the default index ...
+		} elseif( '' != get_settings('permalink_structure') && ! is_admin()) {
+			$permalink = 1;
+			$index = $wp_rewrite->index;
+			// If it's not a path info permalink structure, trim the index.
+			if (! $wp_rewrite->using_index_permalinks()) {
+				$qstr = preg_replace("#/*" . $index . "/*#", '/', $qstr);
+			} else {
+				// If using path info style permalinks, make sure the index is in
+				// the URI.
+				if (strpos($qstr, $index) === false) {
+					$qstr = '/' . $index . $qstr;
+				}
+			}
 
-	 $qstr =  trailingslashit($qstr) . $page_modstring . $pagenum;
-      } else {
-         $qstr = $index . '?' . $page_querystring . '=' . $pagenum;
-      }
-   }
+			$qstr =  trailingslashit($qstr) . $page_modstring . $pagenum;
+		} else {
+			$qstr = $index . '?' . $page_querystring . '=' . $pagenum;
+		}
+	}
 
-   $qstr = preg_replace('|^/+|', '', $qstr);
-   if ($permalink) $qstr = trailingslashit($qstr);
-   return preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', trailingslashit( get_settings('home') ) . $qstr );
+	$qstr = preg_replace('|^/+|', '', $qstr);
+	if ($permalink) $qstr = trailingslashit($qstr);
+	return preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', trailingslashit( get_settings('home') ) . $qstr );
 }
 
 function next_posts($max_page = 0) { // original by cfactor at cooltux.org
@@ -448,12 +425,16 @@ function next_posts($max_page = 0) { // original by cfactor at cooltux.org
 }
 
 function next_posts_link($label='Next Page &raquo;', $max_page=0) {
-    global $paged, $result, $request, $posts_per_page, $wpdb;
+	global $paged, $result, $request, $posts_per_page, $wpdb, $max_num_pages;
     if (!$max_page) {
+			if ( isset($max_num_pages) ) {
+				$max_page = $max_num_pages;
+			} else {
         preg_match('#FROM (.*) GROUP BY#', $request, $matches);
         $fromwhere = $matches[1];
         $numposts = $wpdb->get_var("SELECT COUNT(ID) FROM $fromwhere");
-        $max_page = ceil($numposts / $posts_per_page);
+        $max_page = $max_num_pages = ceil($numposts / $posts_per_page);
+			}
     }
     if (!$paged)
         $paged = 1;
@@ -486,19 +467,21 @@ function previous_posts_link($label='&laquo; Previous Page') {
 }
 
 function posts_nav_link($sep=' &#8212; ', $prelabel='&laquo; Previous Page', $nxtlabel='Next Page &raquo;') {
-	global $request, $posts_per_page, $wpdb;
+	global $request, $posts_per_page, $wpdb, $max_num_pages;
 	if (! is_single()) {
 
 		if (get_query_var('what_to_show') == 'posts') {
-			preg_match('#FROM (.*) GROUP BY#', $request, $matches);
-			$fromwhere = $matches[1];
-			$numposts = $wpdb->get_var("SELECT COUNT(ID) FROM $fromwhere");
-			$max_page = ceil($numposts / $posts_per_page);
+			if ( ! isset($max_num_pages) ) {
+				preg_match('#FROM (.*) GROUP BY#', $request, $matches);
+				$fromwhere = $matches[1];
+				$numposts = $wpdb->get_var("SELECT COUNT(ID) FROM $fromwhere");
+				$max_num_pages = ceil($numposts / $posts_per_page);
+			}
 		} else {
-			$max_page = 999999;
+			$max_num_pages = 999999;
 		}
 
-        if ($max_page > 1) {
+        if ($max_num_pages > 1) {
             previous_posts_link($prelabel);
             echo preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', $sep);
             next_posts_link($nxtlabel, $max_page);
