@@ -523,7 +523,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  logIO('O', 'Post cats: ' . printr($catnames,true));
 	  $post_category = array();
 
-	  if ($catnames) {
+	  if (is_array($catnames)) {
 	    foreach ($catnames as $cat) {
 	      $post_category[] = get_cat_ID($cat);
 	    }
@@ -576,10 +576,12 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $post_content = apply_filters( 'content_save_pre', $content_struct['description'] );
 	  $catnames = $content_struct['categories'];
 		
-	  if ($catnames) {
+	  if (is_array($catnames)) {
 	    foreach ($catnames as $cat) {
 	      $post_category[] = get_cat_ID($cat);
 	    }
+	  } else {
+	    $post_category[] = 1;
 	  }
 
 	  $post_excerpt = $content_struct['mt_excerpt'];
@@ -1160,16 +1162,9 @@ class wp_xmlrpc_server extends IXR_Server {
 		sleep(1);
 
 		// Let's check the remote site
-		$fp = @fopen($pagelinkedfrom, 'r');
-		if (!$fp) {
-			// The source URI does not exist
+		$linea = wp_remote_fopen( $pagelinkedfrom );
+		if ( !$linea )
 	  		return new IXR_Error(16, 'The source URI does not exist.');
-		}
-
-		$puntero = 4096;
-		while($remote_read = fread($fp, $puntero)) {
-			$linea .= $remote_read;
-		}
 
 		// Work around bug in strip_tags():
 		$linea = str_replace('<!DOCTYPE','<DOCTYPE',$linea);
@@ -1177,7 +1172,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		$linea = strip_all_but_one_link($linea, $pagelinkedto);
 		// I don't think we need this? -- emc3
 		//$linea = preg_replace('#&([^amp\;])#is', '&amp;$1', $linea);
-		if (empty($matchtitle)) {
+		if ( empty($matchtitle) ) {
 			preg_match('|<title>([^<]*?)</title>|is', $linea, $matchtitle);
 		}
 		$pos2 = strpos($linea, $pagelinkedto);
